@@ -48,7 +48,6 @@
 #include "got_blame.h"
 #include "got_privsep.h"
 
-#include "proc.h"
 #include "gotwebd.h"
 #include "tmpl.h"
 
@@ -215,7 +214,7 @@ gotweb_process_request(struct request *c)
 			goto err;
 	}
 
-	switch(qs->action) {
+	switch (qs->action) {
 	case BLAME:
 		error = got_get_repo_commits(c, 1);
 		if (error) {
@@ -558,7 +557,7 @@ gotweb_assign_querystring(struct querystring **qs, char *key, char *value)
 	if (error)
 		return error;
 
-	for (el_cnt = 0; el_cnt < QSELEM__MAX; el_cnt++) {
+	for (el_cnt = 0; el_cnt < nitems(querystring_keys); el_cnt++) {
 		if (strcmp(key, querystring_keys[el_cnt].name) != 0)
 			continue;
 
@@ -644,9 +643,10 @@ qa_found:
 			if ((*qs)->page < 0)
 				(*qs)->page = 0;
 			break;
-		default:
-			break;
 		}
+
+		/* entry found */
+		break;
 	}
 done:
 	return error;
@@ -744,6 +744,8 @@ gotweb_free_transport(struct transport *t)
 			free(t->repos[i]);
 		free(t->repos);
 	}
+	if (t->repo)
+		got_repo_close(t->repo);
 	free(t);
 }
 
@@ -868,6 +870,9 @@ gotweb_render_index(struct template *tp)
 
 		r = gotweb_render_repo_fragment(c->tp, repo_dir);
 		gotweb_free_repo_dir(repo_dir);
+		repo_dir = NULL;
+		got_repo_close(t->repo);
+		t->repo = NULL;
 		if (r == -1)
 			return -1;
 
@@ -1103,67 +1108,6 @@ gotweb_render_absolute_url(struct request *c, struct gotweb_url *url)
 	return gotweb_render_url(c, url);
 }
 
-static struct got_repository *
-find_cached_repo(struct server *srv, const char *path)
-{
-	int i;
-
-	for (i = 0; i < srv->ncached_repos; i++) {
-		if (strcmp(srv->cached_repos[i].path, path) == 0)
-			return srv->cached_repos[i].repo;
-	}
-
-	return NULL;
-}
-
-static const struct got_error *
-cache_repo(struct got_repository **new, struct server *srv,
-    struct repo_dir *repo_dir, struct socket *sock)
-{
-	const struct got_error *error = NULL;
-	struct got_repository *repo;
-	struct cached_repo *cr;
-	int evicted = 0;
-
-	if (srv->ncached_repos >= GOTWEBD_REPO_CACHESIZE) {
-		cr = &srv->cached_repos[srv->ncached_repos - 1];
-		error = got_repo_close(cr->repo);
-		memset(cr, 0, sizeof(*cr));
-		srv->ncached_repos--;
-		if (error)
-			return error;
-		memmove(&srv->cached_repos[1], &srv->cached_repos[0],
-		    srv->ncached_repos * sizeof(srv->cached_repos[0]));
-		cr = &srv->cached_repos[0];
-		evicted = 1;
-	} else {
-		cr = &srv->cached_repos[srv->ncached_repos];
-	}
-
-	error = got_repo_open(&repo, repo_dir->path, NULL, sock->pack_fds);
-	if (error) {
-		if (evicted) {
-			memmove(&srv->cached_repos[0], &srv->cached_repos[1],
-			    srv->ncached_repos * sizeof(srv->cached_repos[0]));
-		}
-		return error;
-	}
-
-	if (strlcpy(cr->path, repo_dir->path, sizeof(cr->path))
-	    >= sizeof(cr->path)) {
-		if (evicted) {
-			memmove(&srv->cached_repos[0], &srv->cached_repos[1],
-			    srv->ncached_repos * sizeof(srv->cached_repos[0]));
-		}
-		return got_error(GOT_ERR_NO_SPACE);
-	}
-
-	cr->repo = repo;
-	srv->ncached_repos++;
-	*new = repo;
-	return NULL;
-}
-
 static const struct got_error *
 gotweb_load_got_path(struct request *c, struct repo_dir *repo_dir)
 {
@@ -1171,7 +1115,6 @@ gotweb_load_got_path(struct request *c, struct repo_dir *repo_dir)
 	struct socket *sock = c->sock;
 	struct server *srv = c->srv;
 	struct transport *t = c->t;
-	struct got_repository *repo = NULL;
 	DIR *dt;
 	char *dir_test;
 
@@ -1185,12 +1128,14 @@ gotweb_load_got_path(struct request *c, struct repo_dir *repo_dir)
 	} else {
 		repo_dir->path = dir_test;
 		dir_test = NULL;
-		goto done;
+		goto open_repo;
 	}
 
 	if (asprintf(&dir_test, "%s/%s", srv->repos_path,
-	    repo_dir->name) == -1)
-		return got_error_from_errno("asprintf");
+	    repo_dir->name) == -1) {
+		error = got_error_from_errno("asprintf");
+		goto err;
+	}
 
 	dt = opendir(dir_test);
 	if (dt == NULL) {
@@ -1201,20 +1146,16 @@ gotweb_load_got_path(struct request *c, struct repo_dir *repo_dir)
 		dir_test = NULL;
 	}
 
-done:
+open_repo:
 	if (srv->respect_exportok &&
 	    faccessat(dirfd(dt), "git-daemon-export-ok", F_OK, 0) == -1) {
 		error = got_error_path(repo_dir->name, GOT_ERR_NOT_GIT_REPO);
 		goto err;
 	}
 
-	repo = find_cached_repo(srv, repo_dir->path);
-	if (repo == NULL) {
-		error = cache_repo(&repo, srv, repo_dir, sock);
-		if (error)
-			goto err;
-	}
-	t->repo = repo;
+	error = got_repo_open(&t->repo, repo_dir->path, NULL, sock->pack_fds);
+	if (error)
+		goto err;
 	error = gotweb_get_repo_description(&repo_dir->description, srv,
 	    repo_dir->path, dirfd(dt));
 	if (error)
@@ -1233,6 +1174,10 @@ err:
 	free(dir_test);
 	if (dt != NULL && closedir(dt) == EOF && error == NULL)
 		error = got_error_from_errno("closedir");
+	if (error && t->repo) {
+		got_repo_close(t->repo);
+		t->repo = NULL;
+	}
 	return error;
 }
 
@@ -1352,80 +1297,48 @@ done:
 }
 
 int
-gotweb_render_age(struct template *tp, time_t committer_time, int ref_tm)
+gotweb_render_age(struct template *tp, time_t committer_time)
 {
 	struct request *c = tp->tp_arg;
-	struct tm tm;
 	long long diff_time;
 	const char *years = "years ago", *months = "months ago";
 	const char *weeks = "weeks ago", *days = "days ago";
 	const char *hours = "hours ago",  *minutes = "minutes ago";
 	const char *seconds = "seconds ago", *now = "right now";
-	char *s;
-	char datebuf[64];
-	size_t r;
 
-	switch (ref_tm) {
-	case TM_DIFF:
-		diff_time = time(NULL) - committer_time;
-		if (diff_time > 60 * 60 * 24 * 365 * 2) {
-			if (tp_writef(c->tp, "%lld %s",
-			    (diff_time / 60 / 60 / 24 / 365), years) == -1)
-				return -1;
-		} else if (diff_time > 60 * 60 * 24 * (365 / 12) * 2) {
-			if (tp_writef(c->tp, "%lld %s",
-			    (diff_time / 60 / 60 / 24 / (365 / 12)),
-			    months) == -1)
-				return -1;
-		} else if (diff_time > 60 * 60 * 24 * 7 * 2) {
-			if (tp_writef(c->tp, "%lld %s",
-			    (diff_time / 60 / 60 / 24 / 7), weeks) == -1)
-				return -1;
-		} else if (diff_time > 60 * 60 * 24 * 2) {
-			if (tp_writef(c->tp, "%lld %s",
-			    (diff_time / 60 / 60 / 24), days) == -1)
-				return -1;
-		} else if (diff_time > 60 * 60 * 2) {
-			if (tp_writef(c->tp, "%lld %s",
-			    (diff_time / 60 / 60), hours) == -1)
-				return -1;
-		} else if (diff_time > 60 * 2) {
-			if (tp_writef(c->tp, "%lld %s", (diff_time / 60),
-			    minutes) == -1)
-				return -1;
-		} else if (diff_time > 2) {
-			if (tp_writef(c->tp, "%lld %s", diff_time,
-			    seconds) == -1)
-				return -1;
-		} else {
-			if (tp_writes(tp, now) == -1)
-				return -1;
-		}
-		break;
-	case TM_LONG:
-		if (gmtime_r(&committer_time, &tm) == NULL)
+	diff_time = time(NULL) - committer_time;
+	if (diff_time > 60 * 60 * 24 * 365 * 2) {
+		if (tp_writef(c->tp, "%lld %s",
+		    (diff_time / 60 / 60 / 24 / 365), years) == -1)
 			return -1;
-
-		s = asctime_r(&tm, datebuf);
-		if (s == NULL)
+	} else if (diff_time > 60 * 60 * 24 * (365 / 12) * 2) {
+		if (tp_writef(c->tp, "%lld %s",
+		    (diff_time / 60 / 60 / 24 / (365 / 12)),
+		    months) == -1)
 			return -1;
-
-		if (tp_writes(tp, datebuf) == -1 ||
-		    tp_writes(tp, " UTC") == -1)
+	} else if (diff_time > 60 * 60 * 24 * 7 * 2) {
+		if (tp_writef(c->tp, "%lld %s",
+		    (diff_time / 60 / 60 / 24 / 7), weeks) == -1)
 			return -1;
-		break;
-	case TM_RFC822:
-		if (gmtime_r(&committer_time, &tm) == NULL)
+	} else if (diff_time > 60 * 60 * 24 * 2) {
+		if (tp_writef(c->tp, "%lld %s",
+		    (diff_time / 60 / 60 / 24), days) == -1)
 			return -1;
-
-		r = strftime(datebuf, sizeof(datebuf),
-		    "%a, %d %b %Y %H:%M:%S GMT", &tm);
-		if (r == 0)
+	} else if (diff_time > 60 * 60 * 2) {
+		if (tp_writef(c->tp, "%lld %s",
+		    (diff_time / 60 / 60), hours) == -1)
 			return -1;
-
-		if (tp_writes(tp, datebuf) == -1)
+	} else if (diff_time > 60 * 2) {
+		if (tp_writef(c->tp, "%lld %s", (diff_time / 60),
+		    minutes) == -1)
+			return -1;
+	} else if (diff_time > 2) {
+		if (tp_writef(c->tp, "%lld %s", diff_time,
+		    seconds) == -1)
+			return -1;
+	} else {
+		if (tp_writes(tp, now) == -1)
 			return -1;
-		break;
 	}
 	return 0;
 }
